<?php


namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DTO\Request;

class DadosPessoaisController extends Crud {

    protected $dataSourceName = 'dadosPessoaisCandidato';
    protected $dataSourceName2 = 'dadosPessoaisAdmissao';
    protected $parameterBagColumns = array('NRORG' , 'NRPARCNEGOCIO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag){
        $this->parameterBag = $parameterBag;
    	parent::__construct($dataSourceManager);
    }


    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);  
            $salvar = array();
            foreach ($dataSet->getRows() as $row) {
                if(array_key_exists('DTNASCPESSOA', $row)){
                    $t = str_replace('/', '-', $row['DTNASCPESSOA']);
                    $x = explode(' ', $t);
                    $array = explode('-', $x[0], 3);
                    $row['DTNASCPESSOA'] = date($array[0]."/".$array[1]."/".$array[2]);
                } 
                $salvar[] = $row;       
            }      
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('dadosPessoaisCandidato', $salvar));
        }  catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function salvarDadosAdmissao (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $aux = $this->dataSourceName;
            $this->dataSourceName = $this->dataSourceName2;
            foreach($request->getDataSet()->getRows() as &$row) {
                $dtmescompetencia = DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual());

                if(!array_key_exists('__is_new', $row))
                   $row['__is_new'] = true;

                if(!array_key_exists('IDATIVO', $row))
                  $row['IDATIVO'] = 'S';

                if(!array_key_exists('DTINCLUSAO', $row))
                  $row['DTINCLUSAO'] = date("Y/m/d");

                if(!array_key_exists('DTCASAPESSOA', $row)){
                  $row['DTCASAPESSOA'] = null;
                } else if(array_key_exists('DTCASAPESSOA', $row)){
                    $t = str_replace('/', '-', $row['DTCASAPESSOA']);
                    $x = explode(' ', $t);
                    $array = explode('-', $x[0], 3);
                    $row['DTCASAPESSOA'] = date($array[2]."/".$array[1]."/".$array[0]);
                }

               if(array_key_exists('DTNASCPESSOA', $row)){
                    $t = str_replace('/', '-', $row['DTNASCPESSOA']);
                    $x = explode(' ', $t);
                    $array = explode('-', $x[0], 3);
                    $row['DTNASCPESSOA'] = date($array[2]."/".$array[1]."/".$array[0]);
                }
                /*if (!($row['DTNASCPESSOA'] instanceof \DateTime)) {
                    $row['DTNASCPESSOA'] = DateUtil::getDataDeString($row['DTNASCPESSOA']);
                }*/
                if(array_key_exists('DTNASCPESSOA', $row)){
                    $row['DTNASCPESSOA'] = date($row['DTNASCPESSOA']);
                }
                if(array_key_exists('NRNACIONALIDADE', $row))
                  $row['NRNACIONALID'] = $row['NRNACIONALIDADE'];
            }
            parent::save($request, $response);

            $this->dataSourceName = $aux;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}