<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Util\FPA;

class EstruturasRepository {

    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
    }
    
    //Retorna Nome da legal e gerencial.
    public function getEstruturaLegalGerencial($nrorg, $dtmescompetenc, $conditions) {
        
        $query = $this->entityManager->getConnection()->prepare("
            SELECT * FROM
            (
              SELECT DISTINCT 
              EG.NMESTRUTURAM,
              VH.NRESTRUTLEGAL AS NRESTRUTSUP,
              VH.NRESTRUTGEREN AS NRESTRUTURAM,
              EL.NMESTRUTURAM AS NMESTRUTURASUP
              FROM 
              GPE_VINCULOM VM              
              JOIN GPE_VINCULOH VH ON                 
                VH.NRVINCULOM = VM.NRVINCULOM AND
                VH.NRORG = VM.NRORG AND 
                VH.DTMESCOMPETENC = (SELECT MAX(VHH.DTMESCOMPETENC)
                                             FROM GPE_VINCULOH VHH
                                            WHERE VHH.NRORG      = VH.NRORG
                                              AND VHH.NRVINCULOM = VH.NRVINCULOM
                                              AND VHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
               JOIN ESTRUTURAM EL ON 
                 EL.NRORG = VM.NRORG AND   
                 EL.NRESTRUTURAM = VH.NRESTRUTLEGAL
               JOIN ESTRUTURAM EG ON 
                 EG.NRORG = VM.NRORG AND   
                 EG.NRESTRUTURAM = VH.NRESTRUTGEREN                   
              WHERE VM.NRORG = :P_NRORG    
                AND VH.NRESTRUTGEREN <> VH.NRESTRUTLEGAL
                AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
                AND ( INSTR(','||:P_NRESTRUTLEGALOPER||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGALOPER IS NULL )
                AND ( INSTR(','||:P_NRESTRUTGERENOPER||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :P_NRESTRUTGERENOPER IS NULL )
               
                UNION
                
                SELECT DISTINCT 
                EL.NMESTRUTURAM,
              NULL AS NRESTRUTGEREN,
              VH.NRESTRUTLEGAL AS NRESTRUTURAM,              
              NULL AS NMESTRUTURASUP
              
              FROM 
              GPE_VINCULOM VM              
              JOIN GPE_VINCULOH VH ON                 
                VH.NRVINCULOM = VM.NRVINCULOM AND
                VH.NRORG = VM.NRORG AND 
                VH.DTMESCOMPETENC = (SELECT MAX(VHH.DTMESCOMPETENC)
                                             FROM GPE_VINCULOH VHH
                                            WHERE VHH.NRORG      = VH.NRORG
                                              AND VHH.NRVINCULOM = VH.NRVINCULOM
                                              AND VHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
               JOIN ESTRUTURAM EL ON 
                 EL.NRORG = VM.NRORG AND   
                 EL.NRESTRUTURAM = VH.NRESTRUTLEGAL
               JOIN ESTRUTURAM EG ON 
                 EG.NRORG = VM.NRORG AND   
                 EG.NRESTRUTURAM = VH.NRESTRUTGEREN                   
              WHERE VM.NRORG = :P_NRORG              
                AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
                AND ( INSTR(','||:P_NRESTRUTLEGALOPER||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGALOPER IS NULL )
                AND ( INSTR(','||:P_NRESTRUTGERENOPER||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :P_NRESTRUTGERENOPER IS NULL )
            
            
            )
            WHERE 1=1 
            $conditions
        ");

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_DTMESCOMPETENC"  => $dtmescompetenc,
                "P_NRESTRUTLEGALOPER"  => FPA::getNrestrutlegalOper(),
                "P_NRESTRUTGERENOPER"  => FPA::getNrestrutgerenOper()
            )
        );
        
        $result = $query->fetchAll();
        return $result;
    }
    
    public function retornaEstrutura($nrEstrutura, $nrorg, $dtmescompetenc){
        $params = array('nrorg'=>$nrorg, 'nrestruturam'=>$nrEstrutura, 'dtinivigencia'=>$dtmescompetenc);
        $estruturam = Repositories::ESTRUTURAM;
        
$sql = <<<SQL
SELECT e FROM $estruturam e
WHERE e.nrorg   = :nrorg
  AND e.nrestruturam = :nrestruturam
  AND e.dtinivigencia <= :dtinivigencia
SQL;

        $result = $this->entityManager->createQuery($sql)->execute($params);
        if(!empty($result))
            return $result[0];
        else 
            return $result;
    }
    
    public function retornaEstruturaHistorico($nrEstrutura, $nrorg, $dtmescompetenc){
        
        $params = array('nrorg'=>$nrorg, 'nrestruturam'=>$nrEstrutura, 'dtmescompetenc'=>$dtmescompetenc);
        $estruturah = Repositories::ESTRUTURAH;
        
$sql = <<<SQL
SELECT e FROM $estruturah e
WHERE e.nrorg   = :nrorg
  AND e.nrestruturam = :nrestruturam
  AND e.dtmescompetenc = (
    SELECT MAX(ee.dtmescompetenc) 
        from $estruturah ee
        where ee.nrorg   = :nrorg
        AND ee.nrestruturam = :nrestruturam
        AND ee.dtmescompetenc <= :dtmescompetenc)
SQL;
        $result = $this->entityManager->createQuery($sql)->execute($params);
        
        if(!empty($result))
            return $result[0];
        else 
            return $result;
    }
    
    /**
     * Função responsável por buscar os emails ou o email do responsável informal
     * da estrutura a qual pertence
     */
    public function retornaDadosResponsavelInformal($nrorg, $nrestrutgeren, $dtmescompetenc){
            $sql = "
                SELECT
                        h.nrestruturam,
                        cp.cdcomunicaparc,
                        peh.nmpessoa                       
                    FROM
                        estruturah h                     
                    JOIN
                        comunicaparc cp 
                            on                      cp.nrorg = h.nrorg                        
                            and cp.nrparcnegocio = h.nrparcrespinf                        
                            and CDFORMACOMU = '05'                     
                    JOIN
                        gpe_pessoa pe 
                            on                      pe.nrorg = h.nrorg                          
                            and pe.nrparcnegocio = h.nrparcrespinf                     
                    JOIN
                        gpe_pessoah peh 
                            on  peh.nrorg = pe.nrorg                        
                            and peh.nrpessoa = pe.nrpessoa                        
                            and peh.dtmescompetenc = (
                                SELECT
                                    MAX(DTMESCOMPETENC) 
                            FROM
                                GPE_PESSOAH                                                     
                            WHERE
                                DTMESCOMPETENC <= :p_dtmescompetenc                                                       
                                AND NRORG    = PEH.NRORG                                                       
                                AND NRPESSOA = PEH.NRPESSOA 
                        )                     
                    WHERE
                        h.nrorg = :p_nrorg                        
                        and nrestruturam = :p_nrestrutgeren                        
                        and h.dtmescompetenc = (
                            SELECT
                                MAX(dtmescompetenc)                                                
                            FROM
                                estruturah hh                                               
                            WHERE
                                hh.nrorg = h.nrorg                                                 
                                and hh.nrestruturam = h.nrestruturam                                                 
                                and hh.dtmescompetenc <= :p_dtmescompetenc
                        )

      ";

      $resultado = $this->connection->fetchAll($sql, array(
          "p_nrorg" => $nrorg,
          "p_nrestrutgeren" => $nrestrutgeren,
          "p_dtmescompetenc" => $dtmescompetenc
      ));
      if($resultado)
        return $resultado[0];        
      else
        return $resultado;
    }
    
    public function getEntidadeEducacional($params, $filters){
        
        $query = 'SELECT * 
                    FROM (SELECT ESTRUTHSUP.CDCNPJESTRUT, ESTRUTHSUP.NMRAZSOCESTRUT, ESTRUTHSUP.NMFANTASIA, HIERARQUIAORG.NRHIERARQUIAORG, ESTRUTHSUP.NRORG  
                            FROM HIERARQUIAORG
                       LEFT JOIN ESTRUTURAM ESTRUTREL
                                 ON HIERARQUIAORG.NRESTRUTURAMREL = ESTRUTREL.NRESTRUTURAM
                       LEFT JOIN ESTRUTURAH ESTRUTHSUP
                                 ON ESTRUTREL.NRESTRUTURAM = ESTRUTHSUP.NRESTRUTURAM
                                 AND ESTRUTREL.NRORG        = ESTRUTHSUP.NRORG
                                 AND ESTRUTHSUP.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                      AND NRORG        = ESTRUTREL.NRORG
                                                                      AND NRESTRUTURAM = ESTRUTREL.NRESTRUTURAM )
                       LEFT JOIN ESTRUTURAM ESTRUTSUP
                                 ON HIERARQUIAORG.NRESTRUTURAMSUP = ESTRUTSUP.NRESTRUTURAM
                           WHERE HIERARQUIAORG.NRTIPOHIERARQORG = 71
                                 AND ESTRUTSUP.NRESTRUTURAM = :P_NRESTRUTSUPERIOR
                                 AND ESTRUTREL.NRORG = ESTRUTSUP.NRORG
                                 AND ESTRUTSUP.NRORG = :P_NRORG) A 
                   WHERE (A.NRORG = :P_NRORG OR A.NRORG = :P_NRORG_PADRAO)
                   '.$filters;
        
        
        return $this->connection->fetchAll($query, $params);
    }
    
    public function getEstruturasEducacionaisAdicionar($nrorg, $mesCompetencia, $nrtipoestrutura, $nrhierarquiaorg, $nrestruturasup, $filters){
        
        $query = '
        SELECT * 
        FROM (SELECT EH.*,
                    EM.DTINIVIGENCIA,
                    EM.DTFIMVIGENCIA,
                    EM.NRTIPOESTRUTURA,
                    EM.CDINTESTRUTURA,
                    EM.NMESTRUTURAM,
                    EM.CDINTESTRUTEXPFIL,
                    EM.CDINTESTRUTEXP,
                    TIPOESTRUTURA.NMTIPOESTRUTURA,
                    HIERARQUIAORG.NRHIERARQUIAORG
                FROM ESTRUTURAM EM
                JOIN ESTRUTURAH EH
                    ON EM.NRESTRUTURAM = EH.NRESTRUTURAM
                        AND EM.NRORG = EH.NRORG
                        AND DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC)
                                                FROM ESTRUTURAH EHH
                                               WHERE EHH.NRORG = :P_NRORG
                                                     AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                     AND EHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, \'DD/MM/YYYY\'))
                JOIN TIPOESTRUTURA
                    ON EM.NRTIPOESTRUTURA = TIPOESTRUTURA.NRTIPOESTRUTURA
                LEFT JOIN HIERARQUIAORG 
                    ON HIERARQUIAORG.NRORG = EM.NRORG
                    AND HIERARQUIAORG.NRESTRUTURAMREL = EM.NRESTRUTURAM
                    AND HIERARQUIAORG.NRESTRUTURAMSUP = :P_NRESTRUTURASUP
                WHERE EM.NRORG = :P_NRORG
                AND TIPOESTRUTURA.NRTIPOESTRUTURA = :P_NRTIPOESTRUTURA 
                AND (INSTR(\',\'||:P_NRHIERARQUIAORG||\',\', \',\'||HIERARQUIAORG.NRHIERARQUIAORG||\',\') = 0 OR :P_NRHIERARQUIAORG IS NULL))
        WHERE 1 = 1
        '.$filters;

        $params = array('P_NRORG'            => $nrorg,
                        'P_DTMESCOMPETENC'   => $mesCompetencia,
                        'P_NRTIPOESTRUTURA'  => $nrtipoestrutura,
                        'P_NRHIERARQUIAORG'  => $nrhierarquiaorg,
                        'P_NRESTRUTURASUP' => $nrestruturasup);
        
        $this->connection->beginTransaction();   
        return $this->connection->fetchAll($query, $params);
                        
    } 
    
    public function getEstruturaSuperior($parameters, $conditions){
        $query = "
        SELECT * FROM(            
             SELECT ESTRUTURAH.*,
                    ESTRUTURAM.DTINIVIGENCIA,                
                    ESTRUTURAM.DTFIMVIGENCIA,                       
                    ESTRUTURAM.CDINTESTRUTURA,               
                    ESTRUTURAM.NRESTRUTURAMOLD,             
                    ESTRUTURAM.NMESTRUTURAM,            
                    ESTRUTURAM.DSESTRUTURA,  
                    ESTRUTURAM.CDINTESTRUTEXP,         
                    ESTRUTURAM.CDINTESTRUTEXPFIL,        
                    ESTRUTURAM.CDCENTCUST,         
                    ESTRUTURAM.DSURLLOGOESTRUTURA,     
                    ESTRUTURAM.NRTPEVENTOCTBL,          
                    ESTRUTURAM.NRREGIMEAPUR,
                    ESTRUTURAM.NRTIPOESTRUTURA,
                    TIPOESTRUTURA.NMTIPOESTRUTURA
              FROM ESTRUTURAM
         LEFT JOIN ESTRUTURAH ON ESTRUTURAM.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                             AND ESTRUTURAM.NRORG        = ESTRUTURAH.NRORG
              JOIN TIPOESTRUTURA  ON ESTRUTURAM.NRTIPOESTRUTURA = TIPOESTRUTURA.NRTIPOESTRUTURA 
              JOIN TIPOHIERARQORG ON TIPOESTRUTURA.NRTIPOESTRUTURA = TIPOHIERARQORG.NRTIPOESTRUTSUP
                                 AND TIPOHIERARQORG.NRORG = ESTRUTURAM.NRORG 
             WHERE ESTRUTURAH.NRORG   =  :P_NRORG
               /*AND ESTRUTURAM.IDATIVO = 'S'*/
               AND ( INSTR(','||:P_NRTIPOESTRUTURA||',', ','||TIPOHIERARQORG.NRTIPOESTRUTSUP||',') > 0 OR :P_NRTIPOESTRUTURA IS NULL)
               AND ESTRUTURAH.DTMESCOMPETENC = (SELECT MAX(EH.DTMESCOMPETENC)
                                       FROM ESTRUTURAH EH
                                      WHERE EH.NRORG  =  :P_NRORG
                                        AND EH.NRESTRUTURAM   = ESTRUTURAH.NRESTRUTURAM
                                        AND EH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
               AND (ESTRUTURAM.DTFIMVIGENCIA >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR ESTRUTURAM.DTFIMVIGENCIA IS NULL)
               ORDER BY TIPOESTRUTURA.NMTIPOESTRUTURA, ESTRUTURAH.NMESTRUTURAH)
        WHERE 1=1
        $conditions";
        $params = array("P_NRORG"           => $parameters['NRORG'],
                        "P_NRTIPOESTRUTURA" => $parameters['NRTIPOESTRUTURA'],
                        "P_DTMESCOMPETENC"  => $parameters['DTMESCOMPETENC']);
        
        return $this->connection->fetchAll($query, $params);
        
    }

    
    
}  