<?php
namespace HCM\TRE\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

class TreinamentoRepository {

    protected  $connection;
    
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
    }
   
    public function setDocumento($nrorg,$nrdocumentotre,$nrtreinamento,$nmdocumentotre,$dsrefdocumentotre,$dsrefarquivotre,$idtpdocumentotre,$data,$operador){
        $sql = $this->entityManager->getConnection()->prepare("
            INSERT INTO GPE_DOCUMENTOTRE 
                    (NRORG,NRDOCUMENTOTRE,NRTREINAMENTO,NMDOCUMENTOTRE,DSREFDOCUMENTOTRE,DSREFARQUIVOTRE,IDTPDOCUMENTOTRE,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU)
            VALUES (:NRORG,:NRDOCUMENTOTRE,:NRTREINAMENTO,:NMDOCUMENTOTRE,:DSREFDOCUMENTOTRE,:DSREFARQUIVOTRE,:IDTPDOCUMENTOTRE,:DTINCLUSAO,:DTULTATU,:NRORGINCLUSAO,:CDOPERINCLUSAO,:NRORGULTATU,:CDOPERULTATU)
        ");
        $data = (string)$data;

        $sql->execute(
            array(
                "NRORG" => $nrorg,
                "NRDOCUMENTOTRE" => $nrdocumentotre,
                "NRTREINAMENTO" => $nrtreinamento,
                "NMDOCUMENTOTRE" => $nmdocumentotre,
                "DSREFDOCUMENTOTRE" => $dsrefdocumentotre,
                "DSREFARQUIVOTRE" => $dsrefarquivotre,
                "IDTPDOCUMENTOTRE" => $idtpdocumentotre,
                "DTINCLUSAO" => $data,
                "DTULTATU" => $data,
                "NRORGINCLUSAO" => $nrorg,
                "CDOPERINCLUSAO" => $operador,
                "NRORGULTATU" => $nrorg,
                "CDOPERULTATU" => $operador       
            )
        );
    }
   
    public function getParticipantes($nrorg,$nrtreinamento){
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT A.*,B.NMPRINCIPALPARC 
            FROM GPE_PARTICIPATRE A 
            LEFT JOIN PARCNEGOCIO B 
            ON (A.NRPARCNEGOCIO = B.NRPARCNEGOCIO AND A.NRORG = B.NRORG)  
            WHERE
             A.NRORG = :P_NRORG
            AND A.NRTREINAMENTO = :P_NRTREINAMENTO"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento
                )
            );

        return $sql->fetchAll();
       
   }
   
   public function validaParticipantes($nrorg,$nrtreinamento,$nrparticipatre,$nrparcnegocio){
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT *
            FROM GPE_PARTICIPATRE
            WHERE NRTREINAMENTO = :NRTREINAMENTO
            AND NRPARCNEGOCIO = :NRPARCNEGOCIO
            AND NRORG = :NRORG
            AND NRPARTICIPATRE != :NRPARTICIPATRE
            "
        );

        $sql->execute(
            array(
                "NRTREINAMENTO" => $nrtreinamento,
                "NRPARCNEGOCIO" => $nrparcnegocio,
                "NRORG" => $nrorg,
                "NRPARTICIPATRE" => $nrparticipatre
                )
            );

        $return = $sql->fetchAll();
        return empty($return);
   }
   
    public function getDocumentos($nrorg,$nrtreinamento){
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT * 
             FROM GPE_DOCUMENTOTRE 
            WHERE
             NRORG = :P_NRORG
            AND NRTREINAMENTO = :P_NRTREINAMENTO"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento
                )
            );

        return $sql->fetchAll();
       
   }

    public function getDespesas($nrorg,$nrtreinamento){
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT * 
             FROM GPE_DESPESATRE 
            WHERE
             NRORG = :P_NRORG
            AND NRTREINAMENTO = :P_NRTREINAMENTO"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento
                )
            );
        return $sql->fetchAll();
       
   }

    public function getLocais($nrorg,$nrtreinamento){
       
       $sql = $this->entityManager->getConnection()->prepare(
            "SELECT A.*,B.NMESTRUTURAM 
            FROM GPE_LOCALTREINAMENTO A 
            JOIN ESTRUTURAM B 
            ON (A.NRESTRUTURAM = B.NRESTRUTURAM AND A.NRORG = B.NRORG)
            WHERE (A.NRORG = :P_NRORG AND A.NRTREINAMENTO = :P_NRTREINAMENTO)"
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento
                )
            );
        return $sql->fetchAll();
   }

    public function getMateriais($nrorg,$nrtreinamento){
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT * 
             FROM GPE_RESERVAPRODTRE 
            WHERE
             NRORG = :P_NRORG
            AND NRTREINAMENTO = :P_NRTREINAMENTO"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento
                )
            );

        return $sql->fetchAll();
       
   }

    public function getAvaliacoes($nrorg,$nrtreinamento){
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT * 
             FROM GPE_AVALTREINAMENTO 
            WHERE
             NRORG = :P_NRORG
            AND NRTREINAMENTO = :P_NRTREINAMENTO"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento
                )
            );

        return $sql->fetchAll();
       
   }
    
    public function getOperador($nrorg,$cdoperador){
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT * 
             FROM OPERADOR 
            WHERE
             NRORG = :P_NRORG
            AND CDOPERADOR = :P_CDOPERADOR"
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_CDOPERADOR"=>$cdoperador
                )
            );
        return ($sql->fetchAll()[0]);
   }
   
    public function getParcNegocio($nrorg,$nrparcnegocio){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * 
             FROM PARCNEGOCIO 
            WHERE
             NRORG = :P_NRORG
            AND NRPARCNEGOCIO = :P_NRPARCNEGOCIO"
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRPARCNEGOCIO"=>$nrparcnegocio
                )
        );
        return ($sql->fetchAll()[0]);
    }
    
    public function getCurso($nrorg,$nrcurso){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * 
             FROM GPE_CURSO 
            WHERE
             NRORG = :P_NRORG
            AND NRCURSO = :P_NRCURSO"
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRCURSO"=>$nrcurso
                )
        );
        return ($sql->fetchAll()[0]);
    }
    
    public function getItPerfilCurso($nrorg,$nrcurso){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * 
             FROM GPE_ITPERFILCURSO 
            WHERE
             NRORG = :P_NRORG
             AND NRCURSO = :P_NRCURSO"
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRCURSO"=>$nrcurso
                )
        );
        return($sql->fetchAll());
    }

    public function getParcNegocioByCdoperador($nrorg,$cdoperador){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * 
             FROM PARCNEGOCIO 
            WHERE
             NRORG = :P_NRORG
            AND NRPARCNEGOCIO IN (
                SELECT NRPARCNEGOCIO
                FROM OPERADOR
                WHERE NRORG = :P_NRORG
                AND CDOPERADOR = :P_CDOPERADOR
                )"
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_CDOPERADOR"=>$cdoperador
                )
            );
        return ($sql->fetchAll()[0]);
    }
    
    public function validaDelete($nrorg,$nrtreinamento){
        if(!empty($this->getParticipantes($nrorg,$nrtreinamento))){
            return "Participantes";
        }
        if(!empty($this->getDocumentos($nrorg,$nrtreinamento))){
            return "Documentos";
        }
        if(!empty($this->getDespesas($nrorg,$nrtreinamento))){
            return "Despesas";
        }
        if(!empty($this->getLocais($nrorg,$nrtreinamento))){
            return "Locais";
        }
        if(!empty($this->getMateriais($nrorg,$nrtreinamento))){
            return "Materiaisl";
        }
        if(!empty($this->getAvaliacoes($nrorg,$nrtreinamento))){
            return "Avaliações";
        }
        return true;
    }
    
    public function validaPerfil($nrorg,$nrperfacess,$operador){
        $sql = $this->entityManager->getConnection()->prepare("
            SELECT * 
             FROM OPERADOR 
            WHERE
            NRPARCNEGOCIO IN (
                SELECT NRPARCNEGOCIO 
                FROM PERFACESSPARC
                WHERE NRORG = :P_NRORG
                AND NRPERFILACESSO = :P_PERFACESAPROV
            )
             AND NRORG = :P_NRORG
            AND CDOPERADOR = :P_CDOPERADOR"
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_CDOPERADOR"=> $operador,
                "P_PERFACESAPROV"=>$nrperfacess
                )
        );
        
        if(empty($sql->fetchAll()))
            return false;
        else 
            return true;
   }

    public function aprovaTreinamento($nrorg,$nrtreinamento,$idEstado,$obsAprov,$parceiro,$data){
       $sql = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_TREINAMENTO
            SET IDSTATUSTREINAMENTO = :P_IDSTATUSTREINAMENTO,
            NRPARCNEGAPROV = :P_NRPARCNEGAPROV,
            DTAPROVTREINAMENTO = :P_DTAPROVTREINAMENTO,
            DSOBSAPROVTREINAMENTO = :P_DSOBSAPROVTREINAMENTO
            WHERE NRTREINAMENTO = :P_NRTREINAMENTO
            AND NRORG = :P_NRORG"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento,
                "P_IDSTATUSTREINAMENTO"=>$idEstado,
                "P_NRPARCNEGAPROV"=>$parceiro,
                "P_DSOBSAPROVTREINAMENTO"=>$obsAprov,
                "P_DTAPROVTREINAMENTO"=>$data
                )
            );
   }
   
    public function executaTreinamento($nrorg,$nrtreinamento,$data){
       $sql = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_TREINAMENTO
            SET IDSTATUSTREINAMENTO = 'ANDAMENTO',
            DTINIREALIZADOTRE = :P_DTINIREALIZADOTRE
            WHERE NRTREINAMENTO = :P_NRTREINAMENTO
            AND NRORG = :P_NRORG"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento,
                "P_DTINIREALIZADOTRE"=>$data
                )
            );
   }
   
    public function cancelaTreinamento($nrorg,$nrtreinamento){
       $sql = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_TREINAMENTO
            SET IDSTATUSTREINAMENTO = 'CANCELADO'
            WHERE NRTREINAMENTO = :P_NRTREINAMENTO
            AND NRORG = :P_NRORG"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento
                )
            );
   }
   
    public function atestaEficacia($nrorg,$nrtreinamento,$idEficacia,$obsEficacia,$idValidez){
        $estado = "";
        if(!$idValidez)
            $estado = "IDSTATUSTREINAMENTO = 'INVALIDO',";
       $sql = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_TREINAMENTO
            SET IDEFICACIATREINAMENTO = :P_IDEFICACIATREINAMENTO,"
            .$estado.
            "DSOBSEFICACIATRE = :P_DSOBSEFICACIATRE
            WHERE NRTREINAMENTO = :P_NRTREINAMENTO
            AND NRORG = :P_NRORG"
        );

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento,
                "P_IDEFICACIATREINAMENTO"=>$idEficacia,
                "P_DSOBSEFICACIATRE"=>$obsEficacia
                )
            );
   }
   
    public function concluiTreinamento($nrorg,$nrtreinamento,$data){
       $sql = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_TREINAMENTO
            SET IDSTATUSTREINAMENTO = 'CONCLUIDO',
            DTFIMREALIZADOTRE = :P_DTFIMREALIZADOTRE
            WHERE NRTREINAMENTO = :P_NRTREINAMENTO
            AND NRORG = :P_NRORG"
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento,
                "P_DTFIMREALIZADOTRE"=>$data
                )
            );
   }
   
   public function getParceirosCadastroBasico($nrorg,$data,$nrestrutsind,$nrestrutlegal,$nrestrutgeren,$nrvinculom){
        $where = "";
        if(!empty($nrestrutsind)){
            if(!empty($where))
                $where .= " AND ";
            $where .= " NRESTRUTSIND IN (".join(',',$nrestrutsind).") ";
        };
        if(!empty($nrestrutlegal)){
            if(!empty($where))
                $where .= " AND ";
            $where .= " NRESTRUTLEGAL IN (".join(',',$nrestrutlegal).") ";
        };
        if(!empty($nrestrutgeren)){
            if(!empty($where))
                $where .= " AND ";
            $where .= " NRESTRUTGEREN IN (".join(',',$nrestrutgeren).") ";
        };
        if(!empty($nrvinculom)){
            if(!empty($where))
                $where .= " AND ";
            $where .= " NRVINCULOM IN (".join(',',$nrvinculom).") ";
        };
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT NRPARCNEGOCIO FROM GPE_PESSOA
            WHERE NRORG = :P_NRORG
            AND NRPESSOA IN
                (   SELECT NRPESSOA FROM GPE_VINCULOM 
                    WHERE NRORG = :P_NRORG
                    AND NRVINCULOM IN
                        (SELECT NRVINCULOM FROM(
                            SELECT GPE_VINCULOH.NRVINCULOM,MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                            WHERE (DTMESCOMPETENC <= :P_DTMESCOMPETENC
                            AND NRORG = :P_NRORG) 
                            AND(".$where.")
                            GROUP BY NRVINCULOM
                            ORDER BY NRVINCULOM)
                        )
                    AND (DTRESCISAOVINC IS NULL OR DTRESCISAOVINC > :P_DTMESCOMPETENC)
                )
                     "
        );
        $sql->execute(
            array(
                "P_NRORG" => $nrorg,
                "P_DTMESCOMPETENC" => $data
                )
            );
        return($sql->fetchAll());
   }

    public function insereParticipante($nrorg,$nrtreinamento,$parceiro,$data,$cdoperador){
        $sql = $this->entityManager->getConnection()->prepare("
            SELECT * FROM GPE_PARTICIPATRE 
            WHERE NRORG = :P_NRORG
            AND NRTREINAMENTO = :P_NRTREINAMENTO
            AND NRPARCNEGOCIO = :P_PARCNEGOCIO
            ");
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento,
                "P_PARCNEGOCIO"=>$parceiro
                )
            );
        if(!empty($sql->fetchAll()))
            return false;      
      
      $sql = $this->entityManager->getConnection()->prepare("
            INSERT INTO GPE_PARTICIPATRE
            (NRORG,NRPARTICIPATRE,NRPARCNEGOCIO,NRTREINAMENTO,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU)
            VALUES (:P_NRORG,:P_NRPARTICIPATRE,:P_NRPARCNEGOCIO,:P_NRTREINAMENTO,:P_DTINCLUSAO,:P_DTULTATU,:P_NRORGINCLUSAO,:P_CDOPERINCLUSAO,:P_NRORGULTATU,:P_CDOPERULTATU)
            "
        );
        $sql->execute(
            array(
                ":P_NRORG" => $nrorg,
                ":P_NRPARTICIPATRE" => \HCM\Util\NovoCodigo::geraCodigo( $nrorg, 'GPE_PARTICIPATRE', 1, 12, true),
                ":P_NRPARCNEGOCIO" => $parceiro,
                ":P_NRTREINAMENTO" => $nrtreinamento,
                ":P_DTINCLUSAO" => $data,
                ":P_DTULTATU" => $data,
                ":P_NRORGINCLUSAO" => $nrorg,
                ":P_CDOPERINCLUSAO" => $cdoperador,
                ":P_NRORGULTATU" => $nrorg,
                ":P_CDOPERULTATU" => $cdoperador

                )
            );
        return true;
    }

    public function deletaTodosParticipantes($nrorg,$nrtreinamento){
        $sql = $this->entityManager->getConnection()->prepare("
        DELETE FROM GPE_PARTICIPATRE
        WHERE NRORG = :P_NRORG
        AND NRTREINAMENTO = :P_NRTREINAMENTO
        ");
        $sql->execute(
            array(
                "P_NRORG" => $nrorg,
                "P_NRTREINAMENTO" => $nrtreinamento
                )
            );
        return true;
    }

    public function getPerfilCursoVinculom($nrorg,$nrPerfilCurso,$idPerfil){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * FROM GPE_PERFILCURSOVINC
        WHERE
        NRORG = :P_NRORG
        AND NRPERFILCURSO = :P_NRPERFILCURSO
        AND IDPERFILCURSOVINCULO = :P_IDPERFILCURSO
        ");
        $sql->execute(
            array(
                "P_NRORG" => $nrorg,
                "P_NRPERFILCURSO" => $nrPerfilCurso,
                "P_IDPERFILCURSO" => $idPerfil
                )
            );
        return($sql->fetchAll());
    }

    public function getPerfilCursoEstruturam($nrorg,$nrPerfilCurso,$idPerfil){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * FROM GPE_PERFILCURSOEST
        WHERE
        NRORG = :P_NRORG
        AND NRPERFILCURSO = :P_NRPERFILCURSO
        AND IDPERFILCURSOESTRUT = :P_IDPERFILCURSO
        ");
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRPERFILCURSO"=>$nrPerfilCurso,
                "P_IDPERFILCURSO"=>$idPerfil
                
                )
            );
        return($sql->fetchAll());
    }
    
    public function getPerfilCursoOcupacao($nrorg,$nrPerfilCurso,$idPerfil){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * FROM GPE_PERFILCURSOCUPA
        WHERE
        NRORG = :P_NRORG
        AND NRPERFILCURSO = :P_NRPERFILCURSO
        AND IDPERFILTREOCUPA = :P_IDPERFILCURSO
        ");
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRPERFILCURSO"=>$nrPerfilCurso,
                "P_IDPERFILCURSO"=>$idPerfil
                
                )
            );
        return($sql->fetchAll());
    }
    
    public function getParcNegocioByNrvinculom($nrorg,$nrvinculom,$competencia){
        $sql = $this->entityManager->getConnection()->prepare("
            SELECT NRPARCNEGOCIO FROM GPE_PESSOA
            WHERE NRPESSOA IN(
                SELECT NRPESSOA FROM GPE_VINCULOM
                WHERE NRVINCULOM = :P_NRVINCULOM
                AND (DTRESCISAOVINC IS NULL OR DTRESCISAOVINC > :P_DTMESCOMPETENC)
            )
        AND NRORG = :P_NRORG
        ");
        $sql->execute(
            array(
            "P_NRORG"=>$nrorg,
            "P_NRVINCULOM"=>$nrvinculom,
            "P_DTMESCOMPETENC" => $competencia 
            )
        );
        return($sql->fetchAll()[0]);
    }
        
    public function getParcNegocioByNrestruturam($nrorg,$nrestruturam,$data){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * FROM ESTRUTURAM 
        WHERE NRORG = :P_NRORG
        AND NRESTRUTURAM = :P_NRESTRUTURAM
        ");
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRESTRUTURAM"=>$nrestruturam
                )
        );
        $estruturam = $sql->fetchAll()[0];
        
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * FROM GPE_PARAMORG 
        WHERE NRORG = :P_NRORG");
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg
                )
        );
        $paramorg = $sql->fetchAll()[0];
        switch ($estruturam['NRTIPOESTRUTURA']) {
            case 10:
                $sql = $this->entityManager->getConnection()->prepare("
                    SELECT NRPARCNEGOCIO FROM GPE_PESSOA
                    WHERE NRORG = :P_NRORG
                    AND NRPESSOA IN
                        (SELECT NRPESSOA FROM GPE_VINCULOM 
                        WHERE NRORG = :P_NRORG
                        AND NRVINCULOM IN
                        (SELECT NRVINCULOM FROM(
                                         SELECT GPE_VINCULOH.NRVINCULOM,MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                         WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                         AND NRORG = :P_NRORG
                                         AND NRESTRUTSIND = :P_NRESTRUTURAM
                                         GROUP BY NRVINCULOM
                                         ORDER BY NRVINCULOM)
                                     )
                        AND (DTRESCISAOVINC IS NULL OR DTRESCISAOVINC > :P_DTMESCOMPETENC)
                                     )
                ");
                $sql->execute(
            array(
            "P_NRORG"=>$nrorg,
            "P_NRESTRUTURAM"=>$nrestruturam,
            "P_DTMESCOMPETENC"=>$data
            )
        );
        return($sql->fetchAll());
                break;
            case $paramorg['NRTPESTRUTLEGAL']:
                $sql = $this->entityManager->getConnection()->prepare("
                SELECT NRPARCNEGOCIO FROM GPE_PESSOA
                WHERE NRORG = :P_NRORG
                AND NRPESSOA IN
                    (SELECT NRPESSOA FROM GPE_VINCULOM 
                    WHERE NRORG = :P_NRORG
                    AND NRVINCULOM IN
                    (SELECT NRVINCULOM FROM(
                                     SELECT GPE_VINCULOH.NRVINCULOM,MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                     WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                     AND NRORG = :P_NRORG
                                     AND NRESTRUTLEGAL = :P_NRESTRUTURAM
                                     GROUP BY NRVINCULOM
                                     ORDER BY NRVINCULOM)
                                 )
                    AND (DTRESCISAOVINC IS NULL OR DTRESCISAOVINC > :P_DTMESCOMPETENC)
                                 )
                ");
                $sql->execute(
            array(
            "P_NRORG"=>$nrorg,
            "P_NRESTRUTURAM"=>$nrestruturam,
            "P_DTMESCOMPETENC"=>$data
            )
        );
        return($sql->fetchAll());
                break;
            case $paramorg['NRTPESTRUTGEREN']:
                $sql = $this->entityManager->getConnection()->prepare("
                SELECT NRPARCNEGOCIO FROM GPE_PESSOA
                WHERE NRORG = :P_NRORG
                AND NRPESSOA IN
                    (SELECT NRPESSOA FROM GPE_VINCULOM 
                    WHERE NRORG = :P_NRORG
                    AND NRVINCULOM IN
                    (SELECT NRVINCULOM FROM(
                                     SELECT GPE_VINCULOH.NRVINCULOM,MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                     WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                     AND NRORG = :P_NRORG
                                     AND NRESTRUTGEREN = :P_NRESTRUTURAM 
                                     GROUP BY NRVINCULOM
                                     ORDER BY NRVINCULOM)
                                 )
                    AND (DTRESCISAOVINC IS NULL OR DTRESCISAOVINC > :P_DTMESCOMPETENC)
                                 )
                ");
                $where = " AND NRESTRUTGEREN = :P_NRESTRUTURAM ";
                $sql->execute(
            array(
            "P_NRORG"=>$nrorg,
            "P_NRESTRUTURAM"=>$nrestruturam,
            "P_DTMESCOMPETENC"=>$data
            )
        );
        return($sql->fetchAll());
                break;
            
            default:
                // code...
                break;
        }
    }

    public function getParcNegocioByNrOcupacaom($nrorg,$nrocupacaom,$data){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT NRPARCNEGOCIO FROM GPE_PESSOA
            WHERE NRORG = :P_NRORG
            AND NRPESSOA IN
                (
                SELECT NRPESSOA FROM GPE_VINCULOM 
                WHERE NRORG = :P_NRORG
                AND NRVINCULOM IN
                    (SELECT NRVINCULOM FROM(
                         SELECT GPE_VINCULOH.NRVINCULOM,MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                         WHERE (DTMESCOMPETENC <= :P_DTMESCOMPETENC
                         AND NRORG = :P_NRORG) 
                         AND NRCARGO = :P_NROCUPACAOM
                         GROUP BY NRVINCULOM
                         ORDER BY NRVINCULOM)
                     )
                AND (DTRESCISAOVINC IS NULL OR DTRESCISAOVINC > :P_DTMESCOMPETENC)
                     )
        ");
        $sql->execute(
            array(
            "P_NRORG"=>$nrorg,
            "P_DTMESCOMPETENC"=>$data,
            "P_NROCUPACAOM"=>$nrocupacaom
            )
        );
        return($sql->fetchAll());
    }
    
    public function getParcNegocioByDtValidade($nrorg,$nrcurso,$data,$competencia){
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT P.NRORG, P.NRPARCNEGOCIO, MAX(T.DTVALIDADETREINAMENTO)
        FROM GPE_TREINAMENTO T
        JOIN GPE_PARTICIPATRE P
        ON T.NRORG   = P.NRORG
        AND T.NRTREINAMENTO = P.NRTREINAMENTO
        LEFT JOIN GPE_PESSOA PS
        ON PS.NRPARCNEGOCIO = P.NRPARCNEGOCIO
        AND PS.NRORG = P.NRORG
        LEFT JOIN GPE_VINCULOM VM
        ON VM.NRPESSOA = PS.NRPESSOA
        AND PS.NRORG = P.NRORG
        WHERE T.NRORG = :P_NRORG
        AND T.NRCURSO = :P_NRCURSO
        AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC > :P_DTMESCOMPETENC)
         GROUP BY P.NRORG, P.NRPARCNEGOCIO
        HAVING MAX(T.DTVALIDADETREINAMENTO) < :P_DTVALIDADE
        ");
        $sql->execute(
            array(
                "P_NRORG" => $nrorg,
                "P_NRCURSO" => $nrcurso,
                "P_DTVALIDADE" => $data,
                "P_DTMESCOMPETENC" => $competencia
                )
            );
        return($sql->fetchAll());
        
        
        
    }
    
    public function insereResultadoParticipante($nrorg,$nrtreinamento,$nrpaticipatre,$cdtiporesu){
        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_PARTICIPATRE
            SET CDTIPORESU = :P_CDTIPORESU
            WHERE NRTREINAMENTO = :P_NRTREINAMENTO
            AND NRORG = :P_NRORG
            AND NRPARTICIPATRE = :P_NRPARTICIPATRE
            "
        );
        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRTREINAMENTO"=>$nrtreinamento,
                "P_CDTIPORESU"=>$cdtiporesu,
                "P_NRPARTICIPATRE"=>$nrpaticipatre
                )
            );
   }
   
   public function getFaixaCritResult($nrorg,$nrcritresult){
       $sql = $this->entityManager->getConnection()->prepare("
            SELECT * FROM FAIXACRITRESULT
            WHERE NRORG = :P_NRORG
            AND NRCRITERIORESUL = :P_NRCRITERIORESUL
        ");

        $sql->execute(
            array(
                "P_NRORG"=>$nrorg,
                "P_NRCRITERIORESUL"=>$nrcritresult
                )
            );

        return $sql->fetchAll();
       
   }

}