<?php

namespace HCM\VAB\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\FPA\Services\ExportacaoExcelService;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\FPA;
use Folha\Entities\GpeVinculom;

class ValeTransporteService {
    
    protected $linhatransp;
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->vab_valeTransporte = Repositories::VAB_LINHAVINC;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel = $httpKernel;
    }
    
    public function baixarPlanilhaModelo($nrorg, $nmarquivo) {
        $cabecalho = array(
            "Nr. Linha",
            "Nr. Vínculo",
            "Matrícula",
            "Matrícula eSocial",
            "Qtd. Linhas",
            "Tipo de Linha",
            "Sentido da Linha",
            "Data Inicial",
            "Data Final",
            "Observação"
        );
        $dadosExemplo[0] = array(
            "1058",
            "2023",
            "",
            "",
            "2",
            "208",
            "1",
            "01/01/2021",
            "",
            ""
        );
        
        return $this->exportacaoExcelService->gravarTemplate_1($nrorg, $cabecalho, $nmarquivo, $dadosExemplo);
    }
    
    public function importaArquivoModelo($row) {
        //Dados básicos
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = array(); //Mensages de erro, se estiver vazio não houve erros
        
        //Aquisição dos dados de exportação
        $dados = $this->getDadosPlanilhaModelo($row['DSPATH'][0]);
        foreach($dados as $lin => $dado) {
            $res = $this->inserePlanilhaModelo($lin + 2, $nrorg, $nrorgPadrao,
                $dado['NRLINHATRANSP'], $dado['NRVINCULOM'], $dado['CDMATRICULA'], $dado['CDMATRICULAESOCIAl'], $dado['QTLINHATARIFA'], $dado['NRTIPOLINHAVINC'], $dado['NRSENTIDOLINHA'],
                $dado['DTINILINHAVINC'], $dado['DTFIMLINHAVINC'], $dado['DSOBSLINHAVINC']);
            if(count($res)>0)
                foreach($res as $v)
                    $messages[] = $v; 
        }
        return $messages;
    }
    
    public function getDadosPlanilhaModelo($path) {
        //Os ID's (campos) atribuídos aos dados da planilha
        $head = array(
            'NRLINHATRANSP',
            'NRVINCULOM',
            'CDMATRICULA',
            'CDMATRICULAESOCIAl',
            'QTLINHATARIFA',
            'NRTIPOLINHAVINC',
            'NRSENTIDOLINHA',
            'DTINILINHAVINC',
            'DTFIMLINHAVINC',
            'DSOBSLINHAVINC'
            );
        //Colunas de data
        $dateColumns = array('DTINILINHAVINC', 'DTFIMLINHAVINC');
        
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);
      
        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, array('NRVINCULOM','CDMATRICULA','CDMATRICULAESOCIAl' ), $dateColumns);
    }
    
    public function inserePlanilhaModelo($row, $nrorg, $nrorgPadrao,$nrlinhatransp, $nrvinculom, $cdmatricula, $cdmatriculaesocial,$qtlinhatarifa,$nrtipolinhavinc, $nrsentidolinha, $dtinilinhavinc, $dtfimlinhavinc, $dsobslinhavinc){
        $return = array();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        if($dtinilinhavinc)
            $dtinilinhavinc = DateUtil::getDataDeString($dtinilinhavinc, DateUtil::FORMATO_BRASILEIRO, true);
        if($dtinilinhavinc == null){
            $return[] = array("msg" => "Data inicial inválida, linha $row.");
            return $return;
        }
        if($dtfimlinhavinc) {
            $dtfimlinhavinc = DateUtil::getDataDeString($dtfimlinhavinc, DateUtil::FORMATO_BRASILEIRO, true);
            
            if($dtfimlinhavinc == null){
                $return[] = array("msg" => "Data final inválida, linha $row.");
                return $return;
            }
        }
        try {
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
                array('nrorg'=>$nrorg, 'nrvinculom'=>$nrvinculom));
            if($vinculom == null)
                if($cdmatricula){
                    $objVinculom=GpeVinculom::findOneByNrorgAndCdmatricula($nrorg, $cdmatricula);
                    if ($objVinculom == null)
                        $return[] = array("msg"=>"O código da matrícula do vínculo não foi encontrado na linha $row.");
                    else
                        $nrvinculom = $objVinculom->getNrvinculom();
                }else{
                    if($cdmatriculaesocial){
                        $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true), null);
                        if ($objVinculom == null)
                            $return[] = array("msg"=>"O código da matrícula do ESocial não foi encontrado na linha $row.");
                        else
                            $nrvinculom = $objVinculom->getNrvinculom();
                    }else{
                        $return[] = array("msg"=>"O código do vínculo, da matrícula do vínculo ou da matrícula do ESocial não foram encontrados. É necessário o preenchimento de pelo menos um desses campos.");
                    }
                }
            if($nrlinhatransp){
                $objLinhaVinc = $this->entityManager->getRepository(Repositories::VAB_LINHATARIFA)->findOneBy(array('nrorg'=>$nrorg,'nrlinhatransp'=>$nrlinhatransp));
                if($objLinhaVinc == null)
                    $return[] = array("msg"=>"O número da Linha não existe ou está vazio na linha $row.");
                else
                    $nrlinhatarifa = $objLinhaVinc->getNrlinhatarifa();
            }
            if($nrtipolinhavinc){
                $objnrtipolinhavinc = $this->entityManager->getRepository(Repositories::TIPOLINHAVINC)->findOneBy(array('nrorg'=>array($nrorg, $nrorgPadrao),'nrtipolinhavinc'=>$nrtipolinhavinc));
                if($objnrtipolinhavinc == null)
                $return[] = array("msg"=>"O número do tipo Linha não existe ou está vazio na linha $row.");
            }else
                $return[] = array("msg"=>"O número do tipo Linha não existe ou está vazio na linha $row.");
            if(!is_numeric($qtlinhatarifa) || $qtlinhatarifa < 1){
                $return[] = array("msg"=>"A quantidade de linhas não pode ser vazia na linha $row.");
            }
            if($nrsentidolinha != 1){
                $return[] = array("msg"=>"O valor padrão para o sentido de linha deve ser 1 na linha $row.");
            }
            //VERIFICA SE A LINHA DE TRANSPORTE JA ESTA REGISTRADA PARA O VINCULO
            $objLinhaVinc = $this->entityManager->getRepository(Repositories::VAB_LINHAVINC)->findOneBy(array('nrorg'=>$nrorg, 'nrlinhatarifa'=>$nrlinhatransp, 'nrvinculom'=>$nrvinculom, 'nrtipolinhavinc'=>$nrtipolinhavinc, 'nrsentidolinha'=>$nrsentidolinha));
            if($objLinhaVinc != null){
                $return[] = array("msg"=>"A linha de transporte na linha $row já foi registrada no Sistema.");
            }
            if(count($return)> 0)
                return $return;
            $insereplanilha = $this->nova($nrorg, $nrlinhatransp, $nrvinculom, $qtlinhatarifa, $nrtipolinhavinc, $nrsentidolinha, $dtinilinhavinc, $dtfimlinhavinc, $dsobslinhavinc);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        } 
        return $return;
    }
    
    public function retornaLinhaVinc($row, $nrorg, $nrlinhatarifa, $nrvinculom, $nrtipolinhavinc, $nrsentidolinha){
        $objLinhaVinc = $this->entityManager->getRepository(Repositories::VAB_LINHAVINC)->findBy(['nrorg'=>$nrorg, 'nrlinhatarifa'=>$nrlinhatarifa, 'nrvinculom'=>$nrvinculom, 'nrtipolinhavinc'=>$nrtipolinhavinc, 'nrsentidolinha'=>$nrsentidolinha]);
        
        if($objLinhaVinc){
            foreach($objLinhaVinc as $linhaVinc){
                if(DateUtil::getDataDeString($linhaVinc->getDtinilinhavinc(), DateUtil::FORMATO_BRASILEIRO)->format('d/m/Y') == $row['DTINILINHAVINC']){
                    return true;
                }
            }
        }
    }
    
    public function setReservaValeTransporte($nrorg, $nrVinculom, $idreserva){
        
        $vablinhavincs = $this->entityManager->getRepository(Repositories::VAB_LINHAVINC)->findOneBy(
            [
                'nrorg'      => $nrorg,
                'nrvinculom' => $nrVinculom
            ]
        );
        if($vablinhavincs){
            foreach ($vablinhavincs as $vablinhavinc){
                
                $vabintegtransphs = $this->entityManager->getRepository(Repositories::VAB_INTEGTRANSPH)->findOneBy(
                    [
                        'nrorg'           => $nrorg,                       
                        'nrtipolinhavinc' => $vablinhavinc->getNrtipolinhavinc(),
                        'nrintegtranspm'  => $vablinhavinc->getNrintegtranspm()
                    ]
                );
                
                if($vabintegtransphs){    
                    foreach ($vabintegtransphs as $vabintegtransph){
                        $vabintegtransph->setIdintegreserva($idreserva);
                        $this->entityManager->persist($vabintegtransph);
                        $this->entityManager->flush();
                    }
                }    
            }
        }    
    }
    
    public function setIdReserva($idreserva, $nrintegtranspm){

        $vabintegtransp = $this->entityManager->getRepository(Repositories::VAB_INTEGTRANSPM)->retornaValeTransporte(
            $nrintegtranspm, 
            $this->hcmParameters->getNrorgTrabalho(), 
            $this->hcmParameters->getNrorgPadrao(), 
            $this->hcmParameters->getDtCompetencia()
        );
        
        $vabintegtransph = $vabintegtransp[0]->historico;

        if($vabintegtransph && $this->hcmParameters->getDtCompetencia() == $vabintegtransph->getDtmescompinteg()->format('d/m/Y')){    
            $vabintegtransph->setIdintegreserva($idreserva);

            $vabintegtransph->setDtultatu(Dateutil::getDataDeString($this->hcmParameters->getDataAtual(), DateUtil::FORMATO_BRASILEIRO, false));
            $vabintegtransph->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $vabintegtransph->setCdoperultatu($this->hcmParameters->getCdoperador());

            $this->entityManager->persist($vabintegtransph);
            $this->entityManager->flush();

        }else if( $vabintegtransph && $this->hcmParameters->getDtCompetencia() != $vabintegtransph->getDtmescompinteg()->format('d/m/Y') ){

            $new_vabintegtransph = clone $vabintegtransph;
            
            $new_vabintegtransph->setNrintegtransph( \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_INTEGTRANSPH', 1, 22, true) );
            $new_vabintegtransph->setIdintegreserva($idreserva);
            
            $new_vabintegtransph->setDtmescompinteg(Dateutil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));

            $new_vabintegtransph->setDtinclusao(Dateutil::getDataDeString($this->hcmParameters->getDataAtual(), DateUtil::FORMATO_BRASILEIRO, false));
            $new_vabintegtransph->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $new_vabintegtransph->setCdoperinclusao($this->hcmParameters->getCdoperador());

            $new_vabintegtransph->setDtultatu(Dateutil::getDataDeString($this->hcmParameters->getDataAtual(), DateUtil::FORMATO_BRASILEIRO, false));
            $new_vabintegtransph->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $new_vabintegtransph->setCdoperultatu($this->hcmParameters->getCdoperador());

            $this->entityManager->persist($new_vabintegtransph);
            $this->entityManager->flush();
        }    
        
    }
    
    public function retirarValeDaReserva($nrintegtranspm){
        
        $valeIntTransp = $this->entityManager->getRepository(Repositories::VAB_INTEGTRANSPM)->retornaValeTransporte(
            $nrintegtranspm,
            $this->hcmParameters->getNrorgTrabalho(),
            $this->hcmParameters->getNrorgPadrao(),
            $this->hcmParameters->getDtCompetencia()
        );

        if($valeIntTransp){
            foreach($valeIntTransp as $valeIntTrans){
                $valeIntTrans->historico->setIdintegreserva("N");
                $this->entityManager->persist($valeIntTrans->historico);
                $this->entityManager->flush();
            }
        }        
                
    }
    
    public function nova($nrorg, $nrlinhatransp, $nrvinculom, $qtlinhatarifa,$nrtipolinhavinc, $nrsentidolinha, $dtinilinhavinc, $dtfimlinhavinc, $dsobslinhavinc) {
        try{
            $this->entityManager->beginTransaction();
            $vale = new $this->vab_valeTransporte;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $vale->setNrorg($nrorg);
            $vale->setNrvinculom($nrvinculom);
            $vale->setNrlinhatarifa($nrlinhatransp);
            $vale->setNrtipolinhavinc($nrtipolinhavinc);
            $vale->setNrsentidolinha($nrsentidolinha);
            $vale->setQtlinhatarifa($qtlinhatarifa);
            $vale->setDtinilinhavinc($dtinilinhavinc);
            $vale->setDtfimlinhavinc($dtfimlinhavinc);
            $vale->setNrlinhavinc(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'VAB_LINHAVINC', 1, 12, true));
            $vale->setDsObsLinhaVinc($dsobslinhavinc);
            
            // set auditoria
            $vale->setDtinclusao(DateUtil::getDataAtual());
            $vale->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $vale->setNrorginclusao($nrorg);
            $vale->setDtultatu(DateUtil::getDataAtual());
            $vale->setCdoperultatu($this->hcmParameters->getCdoperador());
            $vale->setNrorgultatu($nrorg);
            $this->entityManager->persist($vale);
            $this->entityManager->flush($vale);
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function novo($row, $nrvinculom) {
        try{
            $this->entityManager->beginTransaction();
            $vale = new $this->vab_valeTransporte;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();

            $diasSemana = $this->validaDiasDaSemana($row);
            $row['IDDIASSEMANA'] = $diasSemana;
            
            $dtinilinhavinc = isset($row['DTINILINHAVINC']) ? DateUtil::getDataDeString($row['DTINILINHAVINC'], DateUtil::FORMATO_BRASILEIRO) : null; 
            $dtfimlinhavinc = isset($row['DTFIMLINHAVINC']) ? DateUtil::getDataDeString($row['DTFIMLINHAVINC'], DateUtil::FORMATO_BRASILEIRO) : null; 
            
            $vale->setNrorg($nrorg);
            $vale->setNrvinculom($nrvinculom);
            $vale->setNrlinhatarifa($row['NRLINHATARIFA']);
            $vale->setNrtipolinhavinc($row['NRTIPOLINHAVINC']);
            $vale->setNrintegtranspm(isset($row['NRINTEGTRANSPM']) && !empty($row['NRINTEGTRANSPM']) ? $row['NRINTEGTRANSPM'] : null);
            $vale->setNrsentidolinha($row['NRSENTIDOLINHA']);
            $vale->setQtlinhatarifa($row['QTLINHATARIFA']);
            $vale->setDtinilinhavinc($dtinilinhavinc);
            $vale->setDtfimlinhavinc($dtfimlinhavinc);
            $vale->setIddiassemana($diasSemana);
            $vale->setNrlinhavinc(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'VAB_LINHAVINC', 1, 12, true));
            
            // set auditoria
            $vale->setDtinclusao(DateUtil::getDataAtual());
            $vale->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $vale->setNrorginclusao($nrorg);
            $vale->setDtultatu(DateUtil::getDataAtual());
            $vale->setCdoperultatu($this->hcmParameters->getCdoperador());
            $vale->setNrorgultatu($nrorg);
            $this->entityManager->persist($vale);
            $this->entityManager->flush($vale);
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function validaDiasDaSemana($row)
    {
        $diasSemanaArray = array();
        array_push($diasSemanaArray, $row['SEGUNDA'], $row['TERCA'], $row['QUARTA'],
            $row['QUINTA'], $row['SEXTA'], $row['SABADO'], $row['DOMINGO']);
    
        if(in_array('S', $diasSemanaArray)) {
            $diasSemana = implode($diasSemanaArray);
            return $diasSemana;
        } else {
            $diasSemana = null;
            return $diasSemana;
        }
    }
} 